//+------------------------------------------------------------------+
//|                                                    RayRobot2.mq4 |
//|  Copyright  2011-12, Marketing Dreams Ltd. All Rights Reserved. |
//|                   http://community.trading-gurus.com/threads/150 |
//|                                                                  |
//| GuruTrader example 7                                            |
//| Version 1.02                                                     |
//|                                                                  |
//| Also known as the Son of Ray the Random Robot, or Ray Robot II |
//| for short, or even just RR2 as a convenient TLA.                |
//|                                                                  |
//| Enters using a pair of stop orders that "bracket" the most       |
//| recent bar, and hence enters on a breakout. Also includes a      |
//| "fade" option to use a pair of limit orders instead.             |
//|                                                                  |
//| Wealth Warning! This expert is for educational purposes only.    |
//| It should NEVER be used on a live account. Past performance is   |
//| in no way indicative of future results!                          |
//+------------------------------------------------------------------+
//+ Revision History                                                 +
//+                                                                  +
//+ 1.02 - January 23rd 2012                                         +
//+        Added Started to allow initial test setup in start()      +
//+        Check for open position on startup                        +
//+        Added order expiry input settings                         +
//+        Added display of startup spread                           +
//+        Removed superfluous Print()s                              +
//+                                                                  +
//+ 1.01 - November 19th 2011                                        +
//+        Included Sunday bar in Monday on D1 timeframe             +
//+        Added TradeAllowed()                                      +
//+                                                                  +
//+ 1.00 - November 11th 2011                                        +
//+        First offspring of Ray the Random Robot to dispense with +
//+        "random" entries.                                         +
//+------------------------------------------------------------------+
#property copyright "Copyright  2011-12, Marketing Dreams Ltd."
#property link      "http://community.trading-gurus.com/threads/150"

#define ROBOT_NAME   "Ray Robot II"
#define VERSION      "v1.02"

#define SLEEP_WAIT   50
#define SLEEP_OK     250
#define SLEEP_ERR    250

#define SECS_PER_MIN  60
#define SECS_PER_HOUR 3600
#define SECS_PER_DAY  86400

#define LONG_UNIT    0
#define SHORT_UNIT   1
#define MAX_ORDERS   2

#define ORDER_NO     0
#define ORDER_STATUS 1
#define MAX_FIELDS   2

#define ORDER_PENDING 1
#define ORDER_FILLED  2
#define ORDER_CLOSED  3

//---- Input parameters
extern int     MMType=0;            // Money management method - 0 = None (Fixed order size)
                                    // 1 = Fixed Fractional, 2 = Fixed % at risk
extern double  Lots = 0.1;          // Size of each order (if not using money management)
extern double  RiskPercent = 1.0;   // Money management master control (use with caution!!!)  
extern bool    FadeEntry = FALSE;   // Set TRUE to use "mean reversion" instead of "breakout" mode
extern int     EntryGap = 1;        // Gap between bar and entry price  
extern int     ProfitTarget = 50;   // Fixed take profit target  
extern int     StopLoss = 10;       // Fixed stop loss            
extern bool    DailyExpiry = false; // Expire every day (for stocks and indices). Otherwise only expire on a Friday.   
extern int     ExpiryHours = 22;    // Expiry period
extern int     ExpiryMins = 59;     //  for pending orders
extern bool    UseLog = true;       // Set TRUE for extra debug messages

//---- static variables
static int     Magic = 12349;
static int     Dig;
static double  Points;
static string  Suffix;
static string  Broker;
static bool    SmartFix = FALSE;

static bool    Initialized = FALSE;
static bool    Running = FALSE;
static bool    Started = FALSE;
static datetime LastTime = 0;       // Time of last bar
static int     StopLevel;
static double  MinLots;
static double  MaxLots;
static double  StepLots;
static double  TickSize;
static int     Spread;
static int     Target;
static int     Stop;
static int     Gap;
static datetime ExpiryTime;         // Order Expiry Time
static string  StrategyName;

static int     Orders[MAX_ORDERS][MAX_FIELDS];
static double  LongEntry;
static double  ShortEntry;
static bool    LongValid = FALSE;
static bool    ShortValid = FALSE;

static color   clBuy = DodgerBlue;
static color   clSell = Crimson;

//+------------------------------------------------------------------+
//| Utility functions                                                |
//+------------------------------------------------------------------+
#include <stdlib.mqh>
#include <stderror.mqh>
#include <WinUser32.mqh>

//+------------------------------------------------------------------+
//| Expert helper functions                                          |
//+------------------------------------------------------------------+

int mod(int a, int b) {

   while (a < 0)
     a += b;
     
   return(a - b * (a / b));
}

//+------------------------------------------------------------------+
//| int TradeAllowed( int MaxWaiting_sec = 30 )
//|
//| the function checks the trade context status. Return codes:
//|  1 - trade context is free, trade allowed
//|  0 - trade context was busy, but became free. Trade is allowed only after 
//|      the market info has been refreshed.
//| -1 - trade context is busy, waiting interrupted by the user (expert was removed from 
//|      the chart, terminal was shut down, the chart period and/or symbol was changed, etc.)
//| -2 - trade context is busy, the waiting limit is reached (MaxWaiting_sec). 
//|      Possibly, the expert is not allowed to trade (checkbox "Allow live trading" 
//|      in the expert settings).
//|
//| MaxWaiting_sec - time (in seconds) within which the function will wait 
//| until the trade context is free (if it is busy). By default,30.
//+------------------------------------------------------------------+
int TradeAllowed(int MaxWaiting_sec = 30)
{
   // check whether the trade context is free
   if(IsConnected() && IsTradeAllowed()) {
      return(1);
   }
   else {
     int StartWaitingTime = GetTickCount();
     
     if (IsConnected())
         Print("Trade context is busy! Wait until it is free...");
     else
         Print("Disconnected! Waiting for reconnection");
     
     while(true)        // infinite loop
       {
         // if the expert was terminated by the user, stop operation
         if(IsStopped()) 
           { 
             Print("The expert was terminated by the user!"); 
             return(-1); 
           }
         // if the waiting time exceeds the time specified in the 
         // MaxWaiting_sec variable, stop operation, as well
         if(GetTickCount() - StartWaitingTime > MaxWaiting_sec * 1000)
           {
             Print("The waiting limit exceeded (" + MaxWaiting_sec + " ???.)!");
             return(-2);
           }
         // if the trade context has become free,
         if(IsConnected() && IsTradeAllowed())
           {
             Print("Trade context has become free!");
             return(0);
           }
         // if no loop breaking condition has been met, "wait" for 0.1 
         // second and then restart checking
         Sleep(SLEEP_WAIT);
       }
   }
}

//+------------------------------------------------------------------+
//| Places a pending order                                           |
//+------------------------------------------------------------------+
int PendingOrder(string symbol, int Type, double Entry, double Quantity, 
          double TargetPrice, double StopPrice, string comment="") 
{
   string TypeStr;
   color  TypeCol;
   int    ErrorCode, Ticket, CheckType;
   double Price, FillPrice;

   if (comment == "")
      comment = StrategyName;
      
   Price = NormalizeDouble(Entry, Dig);
   
   switch (Type) {
      case OP_BUYSTOP: 
         TypeStr = "BUY STOP";
         TypeCol = clBuy;
         CheckType = OP_BUY;
         break; 
      case OP_SELLSTOP: 
         TypeStr = "SELL STOP";
         TypeCol = clSell;
         CheckType = OP_SELL;
         break;
      case OP_BUYLIMIT: 
         TypeStr = "BUY LIMIT";
         TypeCol = clBuy;
         CheckType = OP_BUY;
         break; 
      case OP_SELLLIMIT: 
         TypeStr = "SELL LIMIT";
         TypeCol = clSell;
         CheckType = OP_SELL;
         break;
      default:    
         Print("Unknown pending order type ", Type);
         break; 
   }
      
   if (AccountFreeMarginCheck(Symbol(), CheckType, Quantity) <= 0 || (GetLastError()==134)) {
      Print("Insufficient margin to open ", TypeStr, " order size ", Quantity);
      return (-1);
   }
   
   if (TradeAllowed() < 0)
      return (-1);
      
   Ticket = OrderSend(symbol, Type, Quantity, Price, 0,
              StopPrice, TargetPrice, comment, Magic, 0, TypeCol);
   if (Ticket >= 0) {
      Sleep(SLEEP_OK);
      if (OrderSelect(Ticket, SELECT_BY_TICKET) == FALSE) {
         ErrorCode = GetLastError();
         Print("Error selecting new pending order ", Ticket, ": ",
               ErrorDescription(ErrorCode), " (", ErrorCode, ")");
      }
      return (Ticket);
   } 	
   	
   ErrorCode = GetLastError();
   RefreshRates();
   Print("Error opening ", TypeStr, " order: ", ErrorDescription(ErrorCode),
         " (", ErrorCode, ")", ", Entry = ", Price, ", Target = ",
         TargetPrice, ", Stop = ", StopPrice, ", Current Bid = ", Bid,
         ", Current Ask = ", Ask);
   Sleep(SLEEP_ERR);
	
   return (-1);
}

//+------------------------------------------------------------------+
//| Cancels a pending order                                           |
//+------------------------------------------------------------------+
int Cancel(int Ticket)
{
   int ErrorCode;

   if (OrderSelect(Ticket, SELECT_BY_TICKET) != TRUE) {
      ErrorCode = GetLastError();
      Print("Error selecting order to cancel ", Ticket, ": ",
            ErrorDescription(ErrorCode), " (", ErrorCode, ")");
   }
      
   if (TradeAllowed() < 0)
      return (-1);
      
   if (OrderDelete(Ticket) != TRUE) {
      ErrorCode = GetLastError();
      Print("Error cancelling order ", Ticket, ": ",
            ErrorDescription(ErrorCode), " (", ErrorCode, ")");
      Sleep(SLEEP_ERR);
      return (-1);
   }
   switch(OrderType()) {
   case OP_BUYSTOP:
   case OP_BUYLIMIT:
      Orders[LONG_UNIT][ORDER_STATUS] = ORDER_CLOSED;
      Orders[LONG_UNIT][ORDER_NO] = -1;
      break;
   case OP_SELLSTOP:
   case OP_SELLLIMIT:
      Orders[SHORT_UNIT][ORDER_STATUS] = ORDER_CLOSED;
      Orders[SHORT_UNIT][ORDER_NO] = -1;
      break;
   default:
      break;
   }   
   Sleep(SLEEP_OK);
   return (0);
}
   
//+------------------------------------------------------------------+
//| Modifies an order                                                  |
//+------------------------------------------------------------------+
int Modify(int Ticket, double EntryPrice, double TargetPrice, double StopPrice)
{
   int ErrorCode;
   string TypeStr;
   color  TypeCol;
   double OrderPrice;
   
   if (OrderSelect(Ticket, SELECT_BY_TICKET) != TRUE) {
      ErrorCode = GetLastError();
      Print("Error selecting order ", Ticket, " to modify: ",
            ErrorDescription(ErrorCode), " (", ErrorCode, ")");
      return(-1);
   }

   
   switch (OrderType()) {
       case OP_BUY: 
          TypeStr = "BUY";
          TypeCol = clBuy;
          break; 
       case OP_BUYLIMIT: 
          TypeStr = "BUY Limit";
          TypeCol = clBuy;
          break; 
       case OP_BUYSTOP:
          TypeStr = "BUY Stop";
          TypeCol = clBuy;
          break; 
       case OP_SELL: 
          TypeStr = "SELL";
          TypeCol = clSell;
          break; 
       case OP_SELLLIMIT: 
          TypeStr = "SELL Limit";
          TypeCol = clSell;
          break; 
       case OP_SELLSTOP: 
          TypeStr = "SELL Stop";
          TypeCol = clSell;
          break; 
   }

   if (TradeAllowed() < 0)
      return (-1);

   if (OrderModify(Ticket, NormalizeDouble(EntryPrice, Dig), NormalizeDouble(StopPrice, Dig), NormalizeDouble(TargetPrice, Dig), 0, TypeCol)) {
      Sleep(SLEEP_OK);
      return (Ticket);
   } 	
   	
   ErrorCode = GetLastError();
   RefreshRates();
   Print("Error modifying ", TypeStr, " order ", Ticket,": ", ErrorDescription(ErrorCode),
         " (", ErrorCode, ")", ", Entry = ", NormalizeDouble(EntryPrice, Dig), ", Target = ",
         NormalizeDouble(TargetPrice, Dig), ", Stop = ", NormalizeDouble(StopPrice, Dig),
         ", Current Bid = ", NormalizeDouble(Bid, Dig), ", Current Ask = ", NormalizeDouble(Ask, Dig));
   Sleep(SLEEP_ERR);
	
   return (-1);
}

//+------------------------------------------------------------------+
//| Calculates position size                                         |
//+------------------------------------------------------------------+
double PositionSize()
{
   double Size, TickValue;
   
   if (SmartFix) {         // For Smart Live spread bet use 1/pip
      if ((TickSize == 0.001) || (TickSize == 0.00001))
         TickValue = 0.1;
      else   
         TickValue = 1.0;
   }
   else
      TickValue = MarketInfo(Symbol(), MODE_TICKVALUE);

   switch (MMType) {
   case 1:
      Size = 0.0001 * AccountEquity() * RiskPercent;
      break;
   case 2:
      Size = 0.01 * AccountEquity() * RiskPercent / (Stop * TickValue);
      break;
   default:
      Size = Lots;
      break;
   }
      
   if (Size > MaxLots)
      Size = MaxLots;
   else if (Size < MinLots)
      Size = MinLots;
   else
      Size = MinLots + StepLots * MathFloor((Size - MinLots)/ StepLots);

   if (UseLog) {
      double CreditLine = AccountFreeMargin() * AccountLeverage();
      Print("Account currency = ", AccountCurrency(), ", Free Margin = ", AccountFreeMargin(), 
            ", Leverage = ", AccountLeverage(), ", Credit line = ", AccountCurrency(), " ", CreditLine);
      Print("Margin/lot = ", MarketInfo(Symbol(), MODE_MARGINREQUIRED),
            ", Lot size = ", MarketInfo(Symbol(), MODE_LOTSIZE), ", Tick Value = ", TickValue);
   }
   Print("Money Management Type = ", MMType, ", Size = ", Size);

   return (Size);
}

//+------------------------------------------------------------------+
//| Return true if a new bar appears for the symbol/period pair      |
//+------------------------------------------------------------------+
bool isNewBar()
{
//--- current time 
   datetime lastbar_time = iTime(Symbol(), Period(), 0);
   
//--- if it is the first call of the function
   if(LastTime==0)
     {
      //--- set time and exit
      LastTime=lastbar_time;
      return(false);
     }

//--- if the time is different
   if(LastTime!=lastbar_time)
     {
      //--- memorize time and return true
      LastTime=lastbar_time;
      return(true);
     }
//--- if we pass to this line then the bar is not new, return false
   return(false);
}
  
//+------------------------------------------------------------------+
//| Performs system initialisation                                   |
//+------------------------------------------------------------------+
void InitSystem()
{
   int UnitNumber;
   
   Running = FALSE;
   
   RefreshRates();
   
   for (UnitNumber = 0; UnitNumber < MAX_ORDERS; UnitNumber++) {
      Orders[UnitNumber][ORDER_NO] = -1;
      Orders[UnitNumber][ORDER_STATUS] = ORDER_CLOSED;
   }

   for(int i = 0; i < OrdersTotal(); i++) {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if ((OrderSymbol() == Symbol()) && (OrderMagicNumber() == Magic)) {
         switch(OrderType()) {
         case OP_BUY:
         case OP_SELL:
           Running = TRUE;
           LongValid = FALSE;
           ShortValid = FALSE;
           if (OrderType() == OP_BUY)
             UnitNumber = LONG_UNIT;
           else
             UnitNumber = SHORT_UNIT; 
           Orders[UnitNumber][ORDER_NO] = OrderTicket();
           Orders[UnitNumber][ORDER_STATUS] = ORDER_FILLED;
           Alert("Found open order ", Orders[UnitNumber][ORDER_NO], " for strategy ", StrategyName);
           break; 
         case OP_BUYSTOP:
         case OP_SELLSTOP: 
         case OP_BUYLIMIT:
         case OP_SELLLIMIT: 
            Cancel(OrderTicket());        // Clear the decks, if they're not already!
            break;
         default:    
            Print("Found order ", OrderTicket(), " with unexpected order type ", OrderType());
            break; 
         }
      }
   }
   
   Initialized = TRUE;
}
         
//+------------------------------------------------------------------+
//| Checks for order status changes. Called on each tick             |
//+------------------------------------------------------------------+
int CheckOrders()
{
   int Status;
   int UnitNumber, j;
   double Profit;

   CheckValid();
      
   for (UnitNumber = 0; UnitNumber < MAX_ORDERS; UnitNumber++) {
      if (Orders[UnitNumber][ORDER_STATUS] == ORDER_CLOSED)
         continue;
         
      Status = CheckStatus(Orders[UnitNumber][ORDER_NO]);   // Selects the order
      if (Status > 1) {
         Profit = OrderProfit();
         Print("Order ", Orders[UnitNumber][ORDER_NO], " closed at ", TimeToStr(TimeCurrent(), TIME_SECONDS),
               ", Profit = ", DoubleToStr(Profit, 2));
         Orders[UnitNumber][ORDER_STATUS] = ORDER_CLOSED;
         Orders[UnitNumber][ORDER_NO] = -1;
      }
      else if ((Status > 0) && (Orders[UnitNumber][ORDER_STATUS] < ORDER_FILLED)) {
         Print("Order ", Orders[UnitNumber][ORDER_NO], " filled at ", TimeToStr(TimeCurrent(), TIME_SECONDS));
         Orders[UnitNumber][ORDER_STATUS] = ORDER_FILLED;
         for (j = 0; j < MAX_ORDERS; j++) {
            if (j == UnitNumber)
               continue;
            if (Orders[j][ORDER_STATUS] == ORDER_PENDING) {
               Cancel(Orders[j][ORDER_NO]);
               break;
            }   
         }
      }     
      else if (DailyExpiry || (TimeDayOfWeek(TimeCurrent()) == 5)) {
         if ((ExpiryHours < 24) && (TimeCurrent() >= ExpiryTime) && (Orders[UnitNumber][ORDER_STATUS] < ORDER_FILLED)) {
            Print("Order ", Orders[UnitNumber][ORDER_NO], " expired at ", TimeToStr(TimeCurrent(), TIME_DATE | TIME_SECONDS),
                  ", expiry time ", TimeToStr(ExpiryTime, TIME_DATE | TIME_SECONDS));
            Cancel(Orders[UnitNumber][ORDER_NO]);
         }     
      }
   }
}

//+------------------------------------------------------------------+
//| Checks if latest tick invalidates any entry                      |
//+------------------------------------------------------------------+
void CheckValid()
{
   if (FadeEntry) {
      if (Ask < LongEntry)
         LongValid = false;
      if (Bid > ShortEntry)
         ShortValid = false;
   }
   else {
      if (Ask > LongEntry)
         LongValid = false;
      if (Bid < ShortEntry)
         ShortValid = false;
   }
}

//+------------------------------------------------------------------+
//| Checks for entry to a trade                                      |
//+------------------------------------------------------------------+
bool CheckEntry()
{
   int      UnitNumber;
   int      LongType, ShortType;
   double   PriorHigh, PriorLow;
   datetime BarTime;
   
   for (UnitNumber = 0; UnitNumber < MAX_ORDERS; UnitNumber++) {
      if ((Orders[UnitNumber][ORDER_STATUS] > ORDER_PENDING) && (Orders[UnitNumber][ORDER_STATUS] < ORDER_CLOSED))
         return(true);
   }
   
   PriorHigh = High[1];
   PriorLow = Low[1];
   BarTime = Time[1];
   
   if (Period() == PERIOD_D1) {
      if (TimeDayOfWeek(Time[1]) == 0) {           // "Yesterday" was Sunday?
         PriorHigh = High[2];                      // Use Friday's numbers
         PriorLow = Low[2];
         BarTime = Time[2];
      }
      else if (TimeDayOfWeek(Time[1]) == 1) {      // "Yesterday" was Monday?
         if (TimeDayOfWeek(Time[2]) == 0) {        // and the day before was Sunday?
            PriorHigh = MathMax(High[1], High[2]); // Combine Sunday & Monday's numbers
            PriorLow = MathMin(Low[1], Low[2]);
         }   
      }
   }
   
   if (FadeEntry) {
      LongType = OP_BUYLIMIT;
      ShortType = OP_SELLLIMIT;
      LongEntry = PriorLow + Ask - Bid - Points * Gap;
      ShortEntry = PriorHigh + Points * Gap;
   }
   else {
      LongType = OP_BUYSTOP;
      ShortType = OP_SELLSTOP;
      LongEntry = PriorHigh + Ask - Bid + Points * Gap;
      ShortEntry = PriorLow - Points * Gap;
   }

   ExpiryTime = Time[0] - mod(Time[0], SECS_PER_DAY) + ExpiryHours * SECS_PER_HOUR + ExpiryMins * SECS_PER_MIN;
      
   LongValid = TRUE;
   ShortValid = TRUE;
   CheckValid();
   
   Print("New bar. High: ", DoubleToStr(PriorHigh + Ask - Bid, Dig), ",  Low: ", DoubleToStr(PriorLow, Dig),
         ", Long Valid: ", LongValid, ", Short Valid: ", ShortValid, ", at ", TimeToStr(BarTime));

   if (TimeCurrent() >= ExpiryTime)
      return(false);
      
   if (LongValid) {
      if (Orders[LONG_UNIT][ORDER_STATUS] == ORDER_PENDING) {
         Modify(Orders[LONG_UNIT][ORDER_NO], LongEntry, LongEntry + Points * Target, LongEntry - Points * Stop);
         LongValid = FALSE;
      }
      else {
         Orders[LONG_UNIT][ORDER_NO] = PendingOrder(Symbol(), LongType, LongEntry, PositionSize(),
                  LongEntry + Points * Target, LongEntry - Points * Stop);
         if (Orders[LONG_UNIT][ORDER_NO] > 0) {
            Print("Long entry ", LongEntry, " placed at ", TimeToStr(TimeCurrent()));
            Orders[LONG_UNIT][ORDER_STATUS] = ORDER_PENDING;
            LongValid = FALSE;
         }
      }   
   }
   
   if (ShortValid) {
      if (Orders[SHORT_UNIT][ORDER_STATUS] == ORDER_PENDING) {
         Modify(Orders[SHORT_UNIT][ORDER_NO], ShortEntry, ShortEntry - Points * Target, ShortEntry + Points * Stop);
         ShortValid = FALSE;
      }
      else { 
         Orders[SHORT_UNIT][ORDER_NO] = PendingOrder(Symbol(), ShortType, ShortEntry, PositionSize(),
                  ShortEntry - Points * Target, ShortEntry + Points * Stop);
         if (Orders[SHORT_UNIT][ORDER_NO] > 0) {
            Print("Short entry ", ShortEntry, " placed at ", TimeToStr(TimeCurrent()));
            Orders[SHORT_UNIT][ORDER_STATUS] = ORDER_PENDING;
            ShortValid = FALSE;
         }
      }   
   }
   
   return(false);
}

//+------------------------------------------------------------------+
//| Checks if an order is closed                                     |
//+------------------------------------------------------------------+
int CheckStatus(int Ticket)
{
   int ErrorCode;

   if (OrderSelect(Ticket, SELECT_BY_TICKET) != TRUE) {
      ErrorCode = GetLastError();
      Print("Error selecting order ", Ticket, " to check: ",
            ErrorDescription(ErrorCode), " (", ErrorCode, ")");
      return(-1);
   }
   else if (OrderCloseTime() > 0) {
      Print("Order ", Ticket, " closed: ", DoubleToStr(OrderClosePrice(), Dig),
            ", at ", TimeToStr(OrderCloseTime()));
      return(2);
   }
   else switch(OrderType()) {
   case OP_BUY:
   case OP_SELL:
      return(1);
   default:
      return(0);
   }
}

//+------------------------------------------------------------------+
//| Checks for exit from a trade                                     |
//+------------------------------------------------------------------+
bool CheckExit()
{
   int OpenOrders = 0;
   int Status;
   int UnitNumber, j;
   double Profit;
   
   for (UnitNumber = 0; UnitNumber < MAX_ORDERS; UnitNumber++) {
      if (Orders[UnitNumber][ORDER_STATUS] < ORDER_CLOSED)
         return(false);
   }
   
   Print("All orders closed at ", TimeToStr(TimeCurrent(), TIME_SECONDS));
   
   return(true);
}
   
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
   StrategyName = StringConcatenate(ROBOT_NAME, " ", VERSION);
   
   Dig = MarketInfo(Symbol(), MODE_DIGITS);
   Points = MarketInfo(Symbol(), MODE_POINT);
   TickSize = MarketInfo(Symbol(), MODE_TICKSIZE);
   StopLevel = MarketInfo(Symbol(), MODE_STOPLEVEL);
   MinLots = MarketInfo(Symbol(), MODE_MINLOT);
   MaxLots = MarketInfo(Symbol(), MODE_MAXLOT);
   StepLots = MarketInfo(Symbol(), MODE_LOTSTEP);
   Spread = MarketInfo(Symbol(), MODE_SPREAD);

   Print("Stop Level = ", StopLevel, ", Digits = ", Dig, ", Points = ", DoubleToStr(Points, Dig), ", Spread = ", Spread);
   
   Target = ProfitTarget;
   Stop = StopLoss;
   Gap = EntryGap;
   if ((Dig == 3) || (Dig == 5)) {
      Target *= 10;
      Stop *= 10;
      Gap *= 10;
   }

   if (!IsDemo() && !IsTesting()) {
      MessageBox("Wealth Warning! This expert is for educational purposes only." +
            " It should NEVER be used on a live account." +
            " Past performance is in no way indicative of future results!");
      Print("Initialization Failure");
      return(-1);
   }

   Suffix = StringSubstr(Symbol(), 6);
   Broker = AccountCompany();
   
   if ((Suffix == "SB") && (StringSubstr(Broker, 0, 10) == "Smart Live"))
      SmartFix = TRUE;
         
   LongValid = TRUE;
   ShortValid = TRUE; 

   InitSystem();

   Print("Initialized OK");

   return(0);
}

//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
   for(int i = 0; i < OrdersTotal(); i++) {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if ((OrderSymbol() == Symbol()) && (OrderMagicNumber() == Magic)) {
         switch(OrderType()) {
         case OP_BUY:
         case OP_SELL:
           Alert("Warning - Exiting ", StrategyName, " with order ", OrderTicket(), " still open!");
           break; 
         case OP_BUYSTOP:
         case OP_SELLSTOP: 
         case OP_BUYLIMIT:
         case OP_SELLLIMIT: 
            Cancel(OrderTicket());
            break;
         default:    
            Print("Found order ", OrderTicket(), " with unexpected order type ", OrderType());
            break; 
         }
      }
   }

    Print("DeInitialized OK");
   
    return(0);
}

//+------------------------------------------------------------------+
//| Executed on every new bar                                        |
//+------------------------------------------------------------------+
bool OnBar()
{
   if (Running) {                         // Are we in a trade at the moment?
      if (CheckExit()) {                  // Yes - Last trade complete?
         Initialized = FALSE;             // Yes - Indicate we need to reinitialise
         InitSystem();                    //  and start all over again!
      }
   }
   
   if (!Running)  {
      if (CheckEntry()) {                 // Entered a trade?
         Running = TRUE;                  // Yes - Indicate that we're in a trade
      }   
   }
}

//+------------------------------------------------------------------+
//| Expert start function                                            |
//| Executed on every tick                                           |
//+------------------------------------------------------------------+
int start()
{
   if (!Initialized) {
      return(-1);
   }

   if (!Started) {
      OnBar();                            // Place pending entry orders on startup
      Started = TRUE;
   }

   CheckOrders();

   if (isNewBar()) {
      OnBar();
   }

   return(0);
}

